package com.xdocletbook.blog.ejb;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;

import com.xdocletbook.blog.exception.ApplicationException;
import com.xdocletbook.blog.exception.InvalidUserException;
import com.xdocletbook.blog.interfaces.BlogLocal;
import com.xdocletbook.blog.interfaces.BlogLocalHome;
import com.xdocletbook.blog.interfaces.EntryLocal;
import com.xdocletbook.blog.interfaces.EntryLocalHome;
import com.xdocletbook.blog.interfaces.TopicLocal;
import com.xdocletbook.blog.interfaces.TopicLocalHome;
import com.xdocletbook.blog.util.BlogUtil;
import com.xdocletbook.blog.util.EntryUtil;
import com.xdocletbook.blog.util.TopicUtil;
import com.xdocletbook.blog.value.BlogSimpleValue;
import com.xdocletbook.blog.value.BlogValue;
import com.xdocletbook.blog.value.EntryValue;
import com.xdocletbook.blog.value.TopicValue;

/**
 * A session facade for managing a blog
 * 
 * @ejb.bean 
 *     name="BlogFacade"
 *     type="Stateless"
 *     view-type="remote"
 *     transaction-type="Container"
 * 
 * @ejb.ejb-ref 
 *     ejb-name="Blog" 
 *     view-type="local"
 *     ref-name="ejb/BlogLocal"
 * 
 * @ejb.ejb-ref 
 *     ejb-name="Entry" 
 *     view-type="local"
 *     ref-name="ejb/EntryLocal"
 *
 * @ejb.ejb-ref 
 *     ejb-name="Topic" 
 *     view-type="local"
 *     ref-name="ejb/TopicLocal"
 * 
 * @ejb.security-role-ref
 *     role-name="Owner"
 *     role-link="blogowner"
 */
public abstract class BlogFacadeBean implements SessionBean {

  private SessionContext ctx;
  
  /** @ejb.create-method */
  public void ejbCreate() throws CreateException {
    
  }
  
  public void ejbPostCreate() throws CreateException {
  }
  
  public void setSessionContext(SessionContext ctx)
      throws EJBException, RemoteException {
    this.ctx = ctx;
  }

  // 
  // BLOG METHODS
  //
  
  /** @ejb.interface-method 
   */
  public BlogValue createBlog(String name, String owner, 
      String email) throws ApplicationException {
    try {
      BlogLocalHome home = BlogUtil.getLocalHome();
      BlogLocal blog = home.create(name, owner, email);
      return blog.getBlogValue();
    } catch (NamingException e) {
      throw new ApplicationException(e);
    } catch (CreateException e) {
      throw new ApplicationException(e);
    }
  }
  
  /**
   * @ejb.interface-method
   * @ejb.permission
   *     role-name="Administrator"
   */
  public void deleteBlog(String blogId) 
      throws ApplicationException {
    try {
      BlogUtil.getLocalHome().remove(blogId);
    } catch (NamingException e) {
      throw new ApplicationException(e);
    } catch (RemoveException e) {
      throw new ApplicationException(e);
    }
  }
  
  /**
   * @ejb.interface-method
   */
  public Collection getTopics(String blogId) 
      throws ApplicationException {
    try {
      BlogLocalHome blogHome = BlogUtil.getLocalHome();
      BlogLocal blog = blogHome.findByPrimaryKey(blogId);
      Set blogTopics = blog.getTopics();
      
      List topicList = new ArrayList();
      for(Iterator iter = blogTopics.iterator(); iter.hasNext(); ) {
        TopicLocal topic = (TopicLocal)iter.next();
        topicList.add(topic.getTopicValue());
      }
      
      return topicList;
    } catch (Exception e) {
      throw new ApplicationException(e);
    }
  }
  
  /**
   * @ejb.interface-method
   */
  public BlogSimpleValue getBlogSimple(String blogId) 
      throws ApplicationException {
    try {
      BlogLocalHome home = BlogUtil.getLocalHome();
      BlogLocal blog = home.findByPrimaryKey(blogId);
      return blog.getBlogSimpleValue();
    } catch (Exception e) {
      throw new ApplicationException(e);
    }
  }
  
  // 
  // ENTRY METHODS
  //
  /** 
   * @ejb.interface-method 
   * 
   * @ejb.transaction type="RequiresNew"
   * 
   */
  public EntryValue createEntry(String title, String text, 
      String blogId, String topicId) 
      throws ApplicationException, InvalidUserException {
        
    try {
      BlogLocalHome blogHome = BlogUtil.getLocalHome();
      BlogLocal blogLocal = blogHome.findByPrimaryKey(blogId);
      
      String blogOwner = blogLocal.getOwner();
      String userName = null; // XXX - get user name from context
      
      if(blogOwner.equals(userName)) {
        TopicLocalHome topicHome = TopicUtil.getLocalHome();
        TopicLocal topicLocal = topicHome.findByPrimaryKey(topicId);

        EntryLocalHome entryHome = EntryUtil.getLocalHome();
        EntryLocal entryLocal = 
            entryHome.create(title, text, blogLocal, topicLocal);  
            
        return entryLocal.getEntryValue();
      } else {
        throw new InvalidUserException(userName, blogId);
      }
    } catch (NamingException e) {
      throw new ApplicationException(e);
    } catch (CreateException e) {
      throw new ApplicationException(e);
    } catch (FinderException e) {
      throw new ApplicationException(e);
    }
  }

  /** @ejb.interface-method */
  public List getRecentEntries(String blogId, int count) 
      throws ApplicationException {

    try {
      BlogLocalHome blogHome = BlogUtil.getLocalHome();
      BlogLocal blog = blogHome.findByPrimaryKey(blogId);
      Set blogEntries = blog.getEntries();
      
      List entryList = new ArrayList();
      for(Iterator iter = blogEntries.iterator(); iter.hasNext();) {
        EntryLocal entry = (EntryLocal)iter.next();
        entryList.add(entry.getEntryValue());
      }

      return entryList;
    } catch (NamingException e) {
      throw new ApplicationException(e);
    } catch (FinderException e) {
      throw new ApplicationException(e);
    }
  }
 
  /** @ejb.interface-method */
  public Collection getByBlog(String blogId) 
      throws ApplicationException {
    try {
      BlogLocalHome blogHome = BlogUtil.getLocalHome();
      BlogLocal blog = blogHome.findByPrimaryKey(blogId);

      Date rightNow = new Date();
      Date startDate = new Date(rightNow.getTime() - 60000);
      Date endDate = new Date(rightNow.getTime() + 60000);

      EntryLocalHome entryHome = EntryUtil.getLocalHome();
      Collection allEntries = 
          entryHome.findByBlogWithinDateRange(blog, startDate, 
          endDate);
      
      List entryValues = new ArrayList(allEntries.size());
      for(Iterator iter = allEntries.iterator(); iter.hasNext(); ) {
        EntryLocal entryLocal = (EntryLocal)iter.next();
        entryValues.add(entryLocal.getEntryValue());
      }
      
      return entryValues;
    } catch (NamingException e) {
      throw new ApplicationException(e);
    } catch (FinderException e) {
      throw new ApplicationException(e);
    }
  }  

  /** 
   * @ejb.interface-method 
   */
  public List getAllBlogs() throws ApplicationException {
    try {
      BlogLocalHome blogHome = BlogUtil.getLocalHome();
      Collection allBlogs = blogHome.findAll();
    
      List blogValues = new ArrayList(allBlogs.size());
      for(Iterator iter = allBlogs.iterator(); iter.hasNext(); ) {
        BlogLocal blogLocal = (BlogLocal)iter.next();
        blogValues.add(blogLocal.getBlogSimpleValue());
      }
      
      return blogValues;
    } catch (NamingException e) {
      throw new ApplicationException(e);
    } catch (FinderException e) {
      throw new ApplicationException(e);
    }
  }

  // 
  // TOPIC METHODS
  //
  /** 
   * @ejb.interface-method 
   * 
   * @ejb.permission
   *     role-name="blogowner" 
   */
  public TopicValue createTopic(String name, String blogId) 
      throws ApplicationException, InvalidUserException {
    try {
      BlogLocalHome blogHome = BlogUtil.getLocalHome();
      BlogLocal blog = blogHome.findByPrimaryKey(blogId.trim());

      String userName = ctx.getCallerPrincipal().getName();
      String blogOwner = blog.getOwner();
      if(!userName.equals(blogOwner)) {
        throw new InvalidUserException(userName, blogOwner);
      }
     
      TopicLocalHome home = TopicUtil.getLocalHome();
      TopicLocal topic = home.create(name, blog);

      return topic.getTopicValue();
    } catch (Exception e) {
      throw new ApplicationException(e);
    }
  }
}
