package com.xdocletbook.blog.ejb;

import java.util.Date;
import java.util.Set;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;

import com.xdocletbook.blog.interfaces.BlogLocal;
import com.xdocletbook.blog.interfaces.TopicLocal;
import com.xdocletbook.blog.util.EntryUtil;
import com.xdocletbook.blog.value.EntryValue;

/**
 * Entity bean representing a blog entry.
 * 
 * @ejb.bean
 *     name="Entry"
 *     type="CMP"
 *     cmp-version="2.x"
 *     view-type="local"
 *     primkey-field="id"
 * 
 * @ejb.value-object
 * 
 * @ejb.value-object
 *     name="EntrySimple"
 *     match="simple"
 * 
 * @ejb.finder
 *     signature="java.util.Collection findAll()"
 *     query="SELECT OBJECT(e) FROM Entry AS e"
 * 
 * @ejb.finder
 *     signature="java.util.Collection findByBlogWithinDateRange(
 *         com.xdocletbook.blog.interfaces.BlogLocal blog,
 *         java.util.Date startDate, java.util.Date endDate )"
 *     query="SELECT OBJECT(e) FROM Entry AS e WHERE e.blog = ?1 AND
 *         e.createdDate > ?2 AND e.createdDate < ?3"
 */
public abstract class EntryBean implements EntityBean {
  
  //
  // CREATE METHODS
  //
  /** @ejb.create-method */
  public String ejbCreate(String title, String text, BlogLocal blog, 
      TopicLocal topic) throws CreateException {

    this.setId(EntryUtil.generateGUID(this));
    this.setTitle(title);
    this.setText(text);
    this.setCreatedDate(new Date());
   
    return null;
  }
  
  public void ejbPostCreate(String title, String text, BlogLocal blog,
      TopicLocal topic) throws CreateException {
    
    // Set CMR fields
    this.setBlog(blog);
    this.setTopic(topic);
  }
  
  /** @ejb.create-method */
  public String ejbCreate(EntryValue entryValue) throws CreateException {
    this.setId(EntryUtil.generateGUID(this));
    this.setCreatedDate(new Date());
    this.setEntryValue(entryValue);
    
    return null;
  }

  public void ejbPostCreate(EntryValue entryValue) throws CreateException {
  }
  

  //
  // CMP FIELDS
  //
  /**
   * ID (Primary Key)
   * 
   * @ejb.persistence
   * @ejb.interface-method
   * 
   * @ejb.value-object
   *     match="simple"
   */
  public abstract String getId();
  public abstract void setId(String id);

  /**
   * Blog Entry Title
   * 
   * @ejb.persistence
   * @ejb.interface-method
   * 
   * @ejb.value-object
   *     match="simple"
   */ 
  public abstract String getTitle();

  /** @ejb.interface-method */
  public abstract void setTitle(String title);
  
  /**
   * Blog Entry Text
   * 
   * @ejb.persistence
   * @ejb.interface-method
   */ 
  public abstract String getText();

  /** @ejb.interface-method */
  public abstract void setText(String text);

  /**
   * Blog Time of Creation
   * 
   * @ejb.persistence
   * @ejb.interface-method
   * 
   * @ejb.value-object
   *     match="simple"
   */ 
  public abstract Date getCreatedDate();

  /** @ejb.interface-method */
  public abstract void setCreatedDate(Date createdDate);
  
  //
  // CMR FIELDS
  //
  /**
   * @ejb.interface-method
   * 
   * @ejb.relation
   *     name="blog-entries"
   *     role-name="entry-belongs_to-blog"
   *     cascade-delete="yes"
   */
  public abstract BlogLocal getBlog();
  
  /** @ejb.interface-method */
  public abstract void setBlog(BlogLocal blog);
  

  /**
   * @ejb.interface-method
   * 
   * @ejb.relation
   *     name="topic-entries"
   *     role-name="entry-belongs_to-topic"
   *     cascade-delete="yes"
   */
  public abstract TopicLocal getTopic();
  
  /** @ejb.interface-method */
  public abstract void setTopic(TopicLocal topic);
  
  
  //
  // BUSINESS METHODS
  //
  /** @ejb.interface-method */
  public abstract EntryValue getEntryValue();
  
  /** @ejb.interface-method */
  public void setEntryValue(EntryValue entryValue) {
    this.setTitle(entryValue.getTitle());
    this.setText(entryValue.getText());
  }
}
